############
#Objective
############

#This function is to derive the deviance distribution based on the permutation method
#This function is not to be used independently but will be called by the function seqcutoff()

##########
#INPUTS
##########
#data: the variable 'data' within seqcutoff()
#vslist: a vector of selected variables
#C: the number of permutation times

##########
#OUTPUTS
##########
#dev: a vector of deviance after C permutations (lengthOF this vector is C)



###################################
#Other notes for the function
###################################
# for each permutations, we take the maximum deviance by adding one of the variable in the the variable canadidate list
# for C permutations, all of the maximum deviances form a a new deviance distribution to be used to select the next variable
#in the current candiadate list




update.dev<-function(data,vslist, C,weights,ncores,family){
###calculating weights
    len<-dim(data)[2]
    if(is.null(weights)==TRUE|weights==FALSE){
        w=rep(1,length(data[,len])) ##unweighted version
    } else {  #weighted version
        N1=length(which(data[,len]==1))
        N0=length(which(data[,len]==0))
        if(N1>=N0){
            w=ifelse(data[,len]==1,1,(N1/N0))
        }else{
            w=ifelse(data[,len]==1,(N0/N1),1)
        }
    }



    sel=1:(len-1)
        #which(substring(colnames(data),1,1)=="X") # find the predictors in
                                        #    the dataset
    X=data[,sel]
    y=data[,len]
    N=length(y)
    p=dim(X)[2]
    nonzeroind=which(apply(X,2,sum)!=0) # select predictors not of all zero's
    dev=NULL

    xmat2=data.frame(X[,vslist],y=y)
    mod2=stats::glm(y~.,data=xmat2,family=family,weights=w,control=list(maxit=500))

  ##

  #setup parallel backend to use many processors
  #  cores=parallel::detectCores()
    if(ncores>1){
        cl <- parallel::makeCluster(ncores) #not to overload your computer
        doParallel::registerDoParallel(cl)    
        dev<- foreach::`%dopar%`(foreach::foreach(j=1:C, .combine=cbind),  {

            ind=sample(1:N) #permute index
            XX=X
            yy=y

            if(length(vslist)!=0) XX[,vslist]=X[ind,vslist]

            yy=y[ind]
            ww=w[ind]
            devseq=apply(XX[,setdiff(nonzeroind,vslist)],2,function(x){xmat1=data.frame(XX[,vslist],x,yy=yy);mod1=stats::glm(yy~.,data=xmat1,family=family,weights=ww,control=list(maxit=500));return(summary(mod2)[4]$deviance-summary(mod1)[4]$deviance)})


            dmax=max(devseq)
            dmax
 #dev=c(dev,dmax)
            
        })
  #stop cluster
        dev=as.vector(dev)
        parallel::stopCluster(cl)
    }else{
        dev<-rep(0,C)

        for(j in 1:C){
            ind=sample(1:N) #permute index
            XX=X
            yy=y

            if(length(vslist)!=0) XX[,vslist]=X[ind,vslist]

            yy=y[ind]
            ww=w[ind]
            devseq=apply(XX[,setdiff(nonzeroind,vslist)],2,function(x){xmat1=data.frame(XX[,vslist],x,yy=yy);mod1=stats::glm(yy~.,data=xmat1,family=family,weights=ww,control=list(maxit=500));return(summary(mod2)[4]$deviance-summary(mod1)[4]$deviance)})


            dev[j]=max(devseq)
        }
    }
    return(dev)
}




